clear all
close all

load water

W=pixels;
[N K]=size(W);

%% Produce a random rotation of W
test='unrotated';
if strcmp(test,'rotated'), %% if randomly rotated
  [Qtmp,Rtmp] = qr(randn(N,N));
  W = Qtmp*W;
end

%% number of samples to be used
M=80;
xscale = N:-1:1;

avgW=mean(W,2);
W=W-avgW*ones(1,K);
R=1/K*W*W';
[E,lambda] = eig(R);
Rinv=inv(R);

%% Generate samples from with covariance R
Y = E*sqrt(lambda)*randn([N,M]);
%% Or randomly choose M samples from W
%index=randperm(K);
%Y=W(:,index(1:M));
%figure, imagesc(Y+avgW*ones(1,M))
Rhat = Y*Y'/M;
[lambda,I] = sort(diag(lambda));
E=E(:,I);

%% SMT analysis  %%%%%
fprintf(1,'1. Computing SMT estimator...')
[ESMT,lambdaSMT,SMTArray] = SMTCovarEst(Y);
%[ESMT,lambdaSMT,SMTArray] = SMTCovarEst(Y,2*N);  % with a pre-specified rotation # (2N)
lambdaSMT = diag(lambdaSMT);
lambdaSMT_var = diag(ESMT'*R*ESMT);
dSMT=(sum(lambdaSMT_var./(lambdaSMT))-N)/2+1/2*sum(log(lambdaSMT./lambda))
RSMTinv=ESMT*diag(lambdaSMT.^(-1))*ESMT';

%%% SMT Shrinkage estimator %%%%
fprintf(1,'2. Computing SMT Shrinkage estimator...')
if strcmp(test,'rotated'),
     alpha=0.1;  %% if randomly rotated
else
     alpha=0.8;   %% good for un-rotated
end
%[ESMTshrk,lambdaSMTshrk]=SMTShrk(Y,alpha);   % with a pre-specified alpha
[ESMTshrk,lambdaSMTshrk]=SMTShrk(Y);     % with an estimated alpha
lambdaSMTshrk = diag(lambdaSMTshrk);
lambdaSMTshrk_var = diag(ESMTshrk'*R*ESMTshrk);
dSMTshrk=(sum(lambdaSMTshrk_var./(lambdaSMTshrk))-N)/2+1/2*sum(log(lambdaSMTshrk./lambda))
RSMTshrkinv=ESMTshrk*diag(lambdaSMTshrk.^(-1))*ESMTshrk';


%%% Shrinkage estimator (LOOC) %%%%
fprintf(1,'3. Computing Shrinkage estimator...')
if strcmp(test,'rotated'), 
     alpha=0.001;  %% if randomly rotated 
else
     alpha=0.07;   %% good for un-rotated
end
%[Ehat,lambdahat] = LOOC(Y,alpha);    % with a pre-specified alpha
[Ehat,lambdahat] = LOOC(Y);  % with an estimated alpha
err = sum(sum(abs(Ehat*lambdahat*Ehat' - Rhat)));
lambdahat = diag(lambdahat);
lambdahat_var = diag(Ehat'*R*Ehat);
dhat=(sum(lambdahat_var./(lambdahat))-N)/2+1/2*sum(log(lambdahat./lambda))
Rhatinv=Ehat*diag(lambdahat.^(-1))*Ehat';

figure,set(gcf,'DefaultAxesFontSize',16)
semilogy(xscale, lambda,'-', xscale,lambdahat,'-.', xscale, lambdaSMT,'--', xscale,lambdaSMTshrk,'--','LineWidth',2)
xlabel('index')
ylabel('eigenvalues')
title('Eigenvalue estimation')
legend('True Eigenvalues','Shrinkage est','SMT Estimator','SMT-Shrinkage est')

figure,set(gcf,'DefaultAxesFontSize',16)
semilogy(xscale, lambda,'-', xscale, lambdahat_var,'-.', xscale, lambdaSMT_var,'--',xscale, lambdaSMTshrk_var,'--','LineWidth',2)
xlabel('index')
ylabel('eigenvalues')
title('Variance along eigenvector dimensions')
legend('True Eigenvalues','Shrinkage est', 'SMT Estimator','SMT-Shrinkage est')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SNR for random targets
nTrials=200;
snrtru=zeros(nTrials,2);
snrhat=zeros(nTrials,2);
snrsmt=zeros(nTrials,2);
snrSMTshrk=zeros(nTrials,2);
delSMT=zeros(nTrials,2);
delSMTshrk=zeros(nTrials,2);
xscale=1:nTrials;
for sparse=(0:1),
  
  if (sparse > 0)
    fprintf(1,'Sparse:\n');
  else
    fprintf(1,'Dense:\n');
  end
  
  for count=(1:nTrials),
     t=rand(N,1);
     if (sparse>0),
       t(find(t<0.9))=0; %% sparse target signature
     end
    
     t=t/norm(t);
    
     [snrtru(count,sparse+1)] = mfSNR(R,t,Rinv*t);
     [snrhat(count,sparse+1)] = mfSNR(R,t,Rhatinv*t);
     [snrsmt(count,sparse+1)] = mfSNR(R,t,RSMTinv*t);
    
     [snrSMTshrk(count,sparse+1)] = mfSNR(R,t,RSMTshrkinv*t);
 
   end
    
    avghat = sum(snrhat(:,sparse+1)./snrtru(:,sparse+1))/nTrials;
    avgsmt = sum(snrsmt(:,sparse+1)./snrtru(:,sparse+1))/nTrials;
    delSMT(:,sparse+1) = (snrsmt(:,sparse+1) - snrhat(:,sparse+1))./snrtru(:,sparse+1);
    avgdelSMT = sum(delSMT(:,sparse+1))/nTrials;
    vardelSMT = sum(delSMT(:,sparse+1).^2)/nTrials;
    avgSMTshrk = sum(snrSMTshrk(:,sparse+1)./snrtru(:,sparse+1))/nTrials;
    delSMTshrk(:,sparse+1) = (snrSMTshrk(:,sparse+1) - snrhat(:,sparse+1))./snrtru(:,sparse+1);
    avgdelSMTshrk = sum(delSMTshrk(:,sparse+1))/nTrials;
    vardelSMTshrk = sum(delSMTshrk(:,sparse+1).^2)/nTrials;

    fprintf(1,'AvgSNRR: SE=%6.3f, SMT=%6.3f, delta = %6.3f +/- %5.3f, SMT-S=%6.3f, delta = %6.3f +/- %5.3f\n\n',...
          avghat,avgsmt,avgdelSMT,sqrt(vardelSMT - avgdelSMT*avgdelSMT),...
          avgSMTshrk,avgdelSMTshrk,sqrt(vardelSMTshrk - avgdelSMTshrk*avgdelSMTshrk));

end
